@@ -4,7 +4,7 @@ context("Results for FOCUS D established in expertise for UBA (Ranke 2014)")
 
 test_that("Fits without formation fractions are correct for FOCUS D", {
   expect_warning(
-    fit.noff <- mkinfit(SFO_SFO, FOCUS_D, quiet = TRUE),
+    fit.noff <- mkinfit(SFO_SFO, FOCUS_D, quiet = TRUE, test_residuals = TRUE),
     "Shapiro-Wilk")
 
   expect_equal(round(as.numeric(endpoints(fit.noff)$distimes["parent", ]), 2),
